procedure chkimg (image, check)

string	image	=""		{prompt="image or file name"}
string	check	="access"	{prompt="check for (access|noaccess)",
				enum="access|noaccess"}
string	imtype	=""		{prompt="image file format"}
bool	ok	=no		{prompt="result of I/O check?"}
string	imroot	=""		{prompt="image root name"}
string	impath	=""		{prompt="image path name"}
int	imrlen	=0		{prompt="length of image root name"}
int	implen	=0		{prompt="length of image path name"}
int	imglen	=0		{prompt="length of image root+path name"}
string	imrpstr	=""		{prompt="image region/plane"}
int	iostat	=0		{prompt="integer I/O status"}
bool	verbose	=yes		{prompt="verbose error message?"}

# @(#) task chkimg                    Author: R.A. Jansen -- Feb  4 1998
# @(#)
# @(#) task to check  the I/O status of an image,  return the image root
# @(#) name and character length thereof.  The  'check'  switch controls
# @(#) whether the tasks tests for  existence  or  non-existence  of the
# @(#) image.  If 'imtype'  is  non-empty  upon invokation of this task,
# @(#) only images of type 'imtype' will be searched, otherwise the cur-
# @(#) rent  default  image type is  returned.  The special value "file"
# @(#) for 'imtype'  disables splitting  the image name into a root  and
# @(#) extension, allowing to test the existence of a non-image file and
# @(#) return the path and length of its name.  The  'iostat'  parameter
# @(#) returns the I/O status of the image regardless of 'check',  where
# @(#) "0" means that the image does not exist and "1" that it does. The
# @(#) verbose switch allows for an error message  to be returned to the
# @(#) standard output, the value of which depends on 'check'.
# @(#)
# @(#) Uses rjtools tasks: 'cparse' and 'rpparse'
# @(#)
# @(#) Jun 14 2000 -- Added support for multi-extension / N-dim. images.
# @(#) Jul 12 2003 -- Added distinction between path and true root of an
# @(#)                image name.  Added parameters 'impath',  'implen',
# @(#)                and 'implen', and converted a 'stridx' based loop
# @(#)                into a single 'translit' statement with pipes. RAJ.
# @(#) Aug 12 2003 -- Captured the special case 'image=""&&imtyp="file"'
# @(#)                to properly return the system path  to the current
# @(#)                working directory. RAJ.

begin
	string	img, type, cmode, ctyp, cpln
	int	i, j, ilen, tlen
	bool	bok, rpok

#	Initialize variables...
	img	= image
	cmode	= check
	bok	= no
	rpok	= yes
	ilen    = 0
	type	= imtype
	ctyp	= "Image \""
	cpln	= ""

#	Get image format type...
	if ( type == "" ) {
	   type = "."//envget("imtype")
	} else {
	   if ( substr(type,1,1) != "." ) { type = "."//type }
	   if ( type == ".file" ) {
	      type = ""
	      ctyp = "File \""
	   }
	}

#	Initialize and update output task parameters...
	image	= img
	imtype	= type
	ok	= bok
	imroot	= ""
	impath	= ""
	imrlen	= 0
	implen	= 0
	imglen	= 0
	imrpstr	= ""
	iostat	= 0

#	Check whether a plane in a N-dimensional image was specified
#	and/or an image section...
	i = stridx("[",img)
	j = stridx(" ",img)
	if ( j == 0 ) { j = strlen(img) + 1 }
	if ( i > 0 ) {
	   cpln = substr(img,i,j-1)
	   img  = substr(img,1,i-1)
	   # Check whether region string conforms to valid format...
	   rpparse (cpln, verbose=verbose)
	   if (!rpparse.ok) {
	      rpok = no
	      cpln = ""
	   }
	} else {
	   rpparse ("", verbose=verbose)
	}

#	Get image name exclusive of any image extension...
	tlen = strlen(type)
	ilen = strlen(img)
	if ( substr(img,ilen-tlen+1,ilen) == type ) {
	   img  = substr(img,1,ilen-tlen)
	   ilen = ilen - tlen
	}

#	Get I/O status...
	if ( access(img//type) ) {
	   iostat = 1
	   if ( cmode == "access") { bok = yes }
	} else if ( type == "" && img == "" ) {
	   iostat = 1
	   img = "."
	   if ( cmode == "access") { bok = yes }
	} else {
	   iostat = 0
	   if ( cmode == "noaccess" ) { bok = yes }
	}

#	Check whether a symbolic path is present (e.g. "dev$")...
	if ( stridx("$",img) != 0 || ( type == "" && img == "." ) ) {
	   pathnames (img) | translit ("STDIN", "!", " ", del-, col-) | \
		fields ("-", 2, lines="1", quit-, print-) | scan (img)
	   ilen = strlen(img)
	}

#	Split image path and true root name and update task parameters...
	cparse (img, delim="/", nselect=999)
	if ( cparse.nfields > 1 ) {
	   # check whether only a path was specified (imtype="file")...
	   if ( substr(img,ilen,ilen) == "/" ) {
	      imroot = ""
	   } else {
	      imroot = cparse.cfield
	   }
	   tlen = strlen(imroot)
	   imrlen = tlen
	   implen = ilen - tlen
	   impath = substr(img,1,(ilen-tlen))
	   if ( impath == "./" ) {
	      impath = ""
	      implen = implen - 2
	      ilen   = ilen - 2
	   }
	} else {
	   imroot = img
	   impath = ""
	   implen = 0
	   imrlen = ilen
	}
	
	if (!rpok) { bok = no }
	ok = bok
	imglen  = ilen
	imtype  = type
	imrpstr = cpln
	image	= impath//imroot//type//cpln

#	Verbose output on error...
	if ( rpok && !bok && verbose ) {
	   if ( cmode == "access" ) {
	      print ("ERROR: "//ctyp//img//type//"\" not found!")
	   } else {
	      print ("ERROR: "//ctyp//img//type//"\" already exists!")
	   }
	}

end
