procedure borderstat (image, border)

string	image	=""		{prompt="image name"}
int	border	=1		{prompt="width of image border for statistics"}
real	lower	=INDEF		{prompt="lower cutoff for pixel values"}
real	upper	=INDEF		{prompt="upper cutoff for pixel values"}
real	binwidth=0.1		{prompt="bin width of histogram in sigma"}
bool	verbose	=no		{prompt="print results to screen?"}
bool	label	=yes		{prompt="label column headers?"}
int	npix	=0		{prompt="number of image pixels"}
real	mean	=INDEF		{prompt="average pixel value"}
real	midpt	=INDEF		{prompt="midpoint (median) pixel value"}
real	rmode	=INDEF		{prompt="mode of the pixel values"}
real	stddev	=INDEF		{prompt="standard deviation (sigma)"}
real	min	=INDEF		{prompt="minimum pixel value"}
real	max	=INDEF		{prompt="maximum pixel value"}
real	sum	=INDEF		{prompt="sum of all pixels"}

# @(#) task borderstat                Author: R.A. Jansen -- Feb 12 1996
# @(#)
# @(#) task based on 'imstatistics'  to return the image statistics in a
# @(#) 'border'  pixel wide strip  all around  an image  and return  the
# @(#) results as task parameters,  so other tasks  have access to them.
# @(#)
# @(#) Uses 'rjtools' tasks:  'chkimg' and 'rpbuild'
# @(#)
# @(#) Apr 24 2001 -- Complete rewrite of this task:  added support  for
# @(#)                multi-extension / N-dim. images,  got rid of ncol, 
# @(#)                nrow as required parameters,  and  changed  to use
# @(#)                'imstat' directly  rather than calling  'imgstat',
# @(#)                so subsequent calls to these tasks won't conflict.
# @(#) Aug 15 2003 -- Changed task defaults to INDEF rather than 0 to be
# @(#)                consistent with other rjtools tasks.  Cleaned code
# @(#)                and wrote help file. RAJ

begin
	string	cimg, cflds
	real	rhi, rlo, rbw, ravg, rmed, rmod, rsig, rmin, rmax, zlo
	int	bd, nc, nr, x1, x2, y1, y2, ipix, tpix
	bool	new

#	Initialize variables...
	cimg	= image
	bd	= border
	rlo	= lower
	rhi	= upper
	rbw	= binwidth
	ipix	= 0
	ravg	= INDEF
	rmed	= INDEF
	rmod	= INDEF
	rsig	= INDEF
	rmin	= INDEF
	rmax	= INDEF
	cflds	= "npix,mean,midpt,mode,stddev,min,max"
	new	= no

#	(Re)initialize task parameters...
	npix	= 0
	mean	= INDEF
	midpt	= INDEF
	rmode	= INDEF
	stddev	= INDEF
	min	= INDEF
	max	= INDEF
	sum	= INDEF

#	Are we running IRAFv2.12* or an earlier version?
	new = ( release >= "2.12" )

#	Check whether the image exists...
	chkimg (cimg, "access", imtype="", verbose=yes)
	if (!chkimg.ok) { return }
	cimg = chkimg.impath//chkimg.imroot//chkimg.imtype//chkimg.imrpstr

#	Check the image size...
	hselect (cimg, "i_naxis1", "yes") | scan (nc)
	hselect (cimg, "i_naxis2", "yes") | scan (nr)
	x1 = 1
	x2 = nc
	y1 = 1
	y2 = nr

#	Find the minimum value in the input image...
	if ( new ) {
	   imstatistics (cimg, fields="min", lower=INDEF, upper=INDEF,
	      nclip=0, lsigma=5., usigma=5., binwidth=rbw, format=no,
	      cache=yes) | scan (zlo)
	} else {
	   imstatistics (cimg, fields="min", lower=INDEF, upper=INDEF,
	      binwidth=rbw, format=no) | scan (zlo)
	}

#	Create a temporary image and replace all pixels that are not part
#	of the image border by a value that is lower by 1 than the minimum
#	value found in the image...
	if ( access("tmpbrdstt.fits") ) delete ("tmpbrdstt.fits", yes, verify-)
	imcopy (cimg, "tmpbrdstt.fits", verbose=no)
	rpbuild ((x1+bd), (x2-bd), (y1+bd), (y2-bd), pln=0)
	imreplace ("tmpbrdstt.fits"//rpbuild.imrpstr, (zlo-1.), imagin=0.,
		lower=INDEF, upper=INDEF, radius=0.)

#	Now compute the statistics of the image border...
	if ( rlo > zlo ) zlo = rlo
	if ( new ) {
	   imstatistics ("tmpbrdstt.fits", fields=cflds,
		lower=zlo, upper=rhi, nclip=0, lsigma=5.,
		usigma=5., binwidth=rbw, format=no,
		cache=yes) | scan (ipix,ravg,rmed,rmod,rsig,rmin,rmax)
	} else {
	   imstatistics ("tmpbrdstt.fits", fields=cflds,
		lower=zlo, upper=rhi, binw=rbw,
		format=no) | scan (ipix,ravg,rmed,rmod,rsig,rmin,rmax)
	}
	delete ("tmpbrdstt.fits", yes, verify=no)

#	Check whether there where any pixels in the given range...
	if ( ipix == 0) {
	   printf("ERROR: No pixels with values between %g and %g!\n",rlo,rhi)
	   return
	}
	if ( ipix < 2 ) rsig = 0.

#	Return the image statistics...
	image	= cimg
	lower	= rlo
	upper	= rhi
	npix	= ipix
	mean	= ravg
	midpt	= rmed
	rmode	= rmod
	stddev	= rsig
	min	= rmin
	max	= rmax
	sum	= ravg*ipix

#	Print results to the standard output...
	if (verbose) {
	   #print (cimg//":")
	   if (label) {
	      printf("%9s %8s  %8s  %8s  %8s  %8s  %8s  %8s\n", "npix",
		"mean", "midpt", "mode", "stddev", "min", "max", "sum")
	   }
	   printf("%9d %8f  %8f  %8f  %8f  %8f  %8f  %8f\n", ipix, ravg,
		rmed, rmod, rsig, rmin, rmax, (ravg*ipix))
	}

end
