procedure imgstat (image)

string	image	=""		{prompt="image name"}
real	lower	=INDEF		{prompt="lower cutoff for pixel values"}
real	upper	=INDEF		{prompt="upper cutoff for pixel values"}
real	binwidth=0.1		{prompt="bin width of histogram in sigma"}
bool	verbose	=no		{prompt="print results to screen?"}
bool	label	=yes		{prompt="label column headers?"}
int	npix	=0		{prompt="number of image pixels"}
real	mean	=INDEF		{prompt="average pixel value"}
real	midpt	=INDEF		{prompt="midpoint (median) pixel value"}
real	rmode	=INDEF		{prompt="mode of the pixel values"}
real	stddev	=INDEF		{prompt="standard deviation (sigma)"}
real	min	=INDEF		{prompt="minimum pixel value"}
real	max	=INDEF		{prompt="maximum pixel value"}
real	sum	=INDEF		{prompt="sum of all pixels"}

# @(#) task imgstat                   Author: R.A. Jansen -- May 22 1995
# @(#)
# @(#) task based on 'imutil.imstatistics' to return image statistics as
# @(#) task parameters so that they can be accessed by other procedures.
# @(#) Rewrite of an earlier script by M. Franx.
# @(#)
# @(#) Uses 'rjtools' tasks:  'chkimg'
# @(#)
# @(#) Feb  5 1998 -- Replaced redirecting output to a temporary file by
# @(#)                a 'scan' of the output of 'imstatistics'. RAJ.
# @(#) Jun 19 2002 -- Added (temporary) test to distinguish between 2.11
# @(#)                and 2.12 version of 'imstatistics'. RAJ.
# @(#) Aug 10 2003 -- Changed task defaults to INDEF rather than 0 to be
# @(#)                consistent with other rjtools tasks.  Cleaned code
# @(#)                and wrote help file. RAJ
# @(#) Aug 11 2003 -- Fixed bug of release<2.12: should'v been ">=". RAJ

begin
	string	cimg, fields
	real	rlo, rhi, rbw, ravg, rsig, rmin, rmax, rmed, rmod
	int	ipix
	bool	new

#	Initialize variables...
	cimg	= image
	rlo	= lower
	rhi	= upper
	rbw	= binwidth
	ipix	= 0
	ravg	= INDEF
	rmed	= INDEF
	rmod	= INDEF
	rsig	= INDEF
	rmin	= INDEF
	rmax	= INDEF
	fields	= "npix,mean,midpt,mode,stddev,min,max"
	new	= no

#	(Re)initialize task parameters...
	npix	= 0
	mean	= INDEF
	midpt	= INDEF
	rmode	= INDEF
	stddev	= INDEF
	min	= INDEF
	max	= INDEF
	sum	= INDEF

#	Are we running IRAFv2.12* or an earlier version?
	new = ( release >= "2.12" )

#	Check whether the image exists...
	chkimg (cimg, "access", imtype="", verbose=yes)
	if (!chkimg.ok) { return }
	cimg = chkimg.impath//chkimg.imroot//chkimg.imtype//chkimg.imrpstr

#	Run 'imstatistics' ...
	if (new) {
	   imstatistics (cimg, fields=fields, lower=rlo, upper=rhi,
	      nclip=0, lsigma=5., usigma=5., binwidth=rbw, format=no,
	      cache=yes) | scan(ipix,ravg,rmed,rmod,rsig,rmin,rmax)
	} else {
	   imstatistics (cimg, fields=fields, lower=rlo, upper=rhi, binw=rbw, 
	      format=no) | scan(ipix,ravg,rmed,rmod,rsig,rmin,rmax)
	}

#	Check whether there where any pixels in the given range...
	if ( ipix == 0) {
	   printf("ERROR: No pixels with values between %g and %g!\n",rlo,rhi)
	   return
	}
	if ( ipix < 2 ) rsig = 0.

#	Return the image statistics...
	image	= cimg
	lower	= rlo
	upper	= rhi
	npix	= ipix
	mean 	= ravg
	midpt	= rmed
	rmode	= rmod
	stddev	= rsig
	min	= rmin
	max	= rmax
	sum	= ravg*ipix

#	Print results to the standard output...
	if (verbose) {
	   #print (cimg//":")
	   if (label) {
	      printf("%9s %8s  %8s  %8s  %8s  %8s  %8s  %8s\n", "npix",
		"mean", "midpt", "mode", "stddev", "min", "max", "sum")
	   }
           printf("%9d %8f  %8f  %8f  %8f  %8f  %8f  %8f\n", ipix, ravg,
		rmed, rmod, rsig, rmin, rmax, (ravg*ipix))
	}

end
