      PROGRAM arg

C @(#)Program: arg                    Author: R.A. Jansen -- Feb 23 1998
C @(#)
C @(#)program to replace all instances of a user defined marker string
C @(#)(default='&') in a template file by text strings listed in another
C @(#)file. The action taken by this program depends on the value of
C @(#)run-mode switch '-m' as follows:
C @(#)-m0:   loop through file1 and replicate all lines in file2 while
C @(#)       replacing the marker strings by the replacement string --
C @(#)       this is the default action of this task;
C @(#)-m1:   loop through both file1 and file2 one line at a time and
C @(#)       copy each line in file2 while replacing the marker strings
C @(#)       in that line by the replacement string. Note that file1 and
C @(#)       file2 have to be of equal length;
C @(#)-m[n]: loop through file1 one line at a time and through file2 [n]
C @(#)       lines at a time and copy each line per [n] line block in
C @(#)       file2 while replacing the marker strings in that line-block
C @(#)       by the replacement string. Note that file2 needs to be [n]
C @(#)       times the length of file1.
C @(#)Output is to the standard output (i.e. screen) but can be redirec-
C @(#)ted to any other device.
C @(#)
C @(#)Apr 23 2001 -- Compilation from within IRAF no longer necessary;
C @(#)            added run-mode switch '-c' switch to skip commented
C @(#)            lines (starting with a #) in file2 for -m0. -v2.1 RAJ.
C @(#)Nov 27 2001 -- Created system man-page, and replaced the manual
C @(#)            code by a simple read of an external file. - v2.2 RAJ.

      CHARACTER*80 file1, file2, subs
      INTEGER   ierr, ntem, comm

C --------------------------   M  A  I  N   ----------------------------

C     Initialize...
      ierr = 0
      DO i=1,80
         file1(i:i) = ' '
         file2(i:i) = ' '
         subs(i:i) = ' '
      ENDDO

C     Parse command line input...
      CALL getinp(file1,file2,subs,ntem,comm,ierr)
      IF (ierr.NE.0) GOTO 9

C     Substitute marker strings by replacement strings...
      IF (ntem.EQ.0) THEN
         CALL argfilcp(file1,file2,subs,comm,ierr)
      ELSE IF (ntem.EQ.1) THEN
	 CALL arglincp(file1,file2,subs,ierr)
      ELSE
         CALL argblkcp(file1,file2,subs,ntem,ierr)
      ENDIF

9     STOP
      END


C ----------------------------------------------------------------------
C          S U B R O U T I N E S     A N D     F U N C T I O N S
C ----------------------------------------------------------------------

      FUNCTION llen(cstring)

      CHARACTER*(*) cstring
      INTEGER llen,i
     
      DO 1 i=LEN(cstring),1,-1
         llen=i
C         IF (cstring(llen:llen).NE.'') RETURN
         IF (cstring(llen:llen).NE.CHAR(32)) RETURN
1     CONTINUE
      llen=0
      RETURN
      
      END
      

C ----------------------------------------------------------------------

      SUBROUTINE getinp(file1,file2,subs,ntem,comm,ierr)

      CHARACTER*80 file1, file2, subs, cstr(5), ctmp
      INTEGER   ierr, ntem, comm, nargs, llen, i, j, nm, nc

      DO j=1,5
         DO i=1,80
            cstr(j)(i:i) = ' '
         ENDDO
      ENDDO
      ierr = 0
      subs = '&'
      ntem = 0
      comm = 0
      nm   = 0
      nc   = 0

      CALL clnarg (nargs)
      IF (nargs.GE.6) THEN
         WRITE(6,'(A)')'ERROR: too many arguments!'
         CALL manual()
         ierr = 1
         GOTO 9
      ELSE IF (nargs.EQ.0) THEN
         WRITE(6,'(A)')'ERROR: no arguments!'
         CALL manual()
         ierr = 1   
         GOTO 9
      ELSE IF (nargs.EQ.1.OR.nargs.EQ.2) THEN
         WRITE(6,'(A)')'ERROR: missing arguments!'
         CALL manual()  
         ierr = 1
         GOTO 9
      ELSE
C        Allowed number of arguments; read input...
	 DO i=1,nargs
            CALL clargs(i,cstr(i),ierr)
            IF (ierr.NE.0) GOTO 8
         ENDDO

C	 Check whether action switch '-m' of task is default, and check
C	 whether to skip commented lines in the template (default mode,
C	 '-m0' only)...
	 DO i=1,nargs
           IF (cstr(i)(1:2).EQ.'-m') THEN
	     nm = i
             ctmp = ' '//cstr(i)(3:llen(cstr(i)))//' '
             READ(ctmp,*,IOSTAT=ierr) ntem
	     IF (ierr.NE.0) GOTO 7
           ENDIF
	   IF (cstr(i)(1:2).EQ.'-c') THEN
             nc   = i
             comm = 1
           ENDIF
         ENDDO

         IF (ntem.GT.0.AND.comm.EQ.1) THEN
            WRITE(6,'(A)')'NOTE: skipping of commented lines in <file'//
     +        '2> is only supported in'
            WRITE(6,'(A)')'      default mode (-m0). Ignoring \"-c\" '//
     +        'switch ...'
         ENDIF

C	 Read from remaining strings 'file1', 'file2' and 'subs'...
         j = 0
         DO i=1,nargs
            IF (i.NE.nm.AND.i.NE.nc) THEN
               j = j + 1
               IF (j.EQ.1) THEN
                  file1 = cstr(i)(1:llen(cstr(i)))
                  IF (llen(file1).EQ.0) GOTO 7
	       ELSE IF (j.EQ.2) THEN
                  file2 = cstr(i)(1:llen(cstr(i)))
                  IF (llen(file2).EQ.0) GOTO 7
               ELSE IF (j.EQ.3) THEN
                  subs = cstr(i)(1:llen(cstr(i)))
                  IF (llen(subs).EQ.0) GOTO 7
               ELSE
                  GOTO 7
               ENDIF
            ENDIF
	 ENDDO

      ENDIF
      GOTO 9

C     Error handling...
7     WRITE(6,'(A)')'ERROR: parsing command line input!'
      ierr = 1
      GOTO 9
8     WRITE(6,'(A)')'ERROR: while reading command line input!'

9     RETURN
      END


C ----------------------------------------------------------------------

      SUBROUTINE argfilcp(file1,file2,subs,comm,ierr)

C @(#)subroutine argfilcp: loop over the lines with replacement strings
C @(#)        in file1 and replicate all lines in file2 while replacing 
C @(#)        the marker strings 'subs' by the replacement strings.
      INTEGER   ierr, comm, llen, ns, i,j, nc1, nc2
      CHARACTER*80 file1, file2, subs
      CHARACTER*1024 cstr1, cstr2, newstr

C     Initialize...
      DO i=1,1024
         cstr1(i:i)  = ' '
         cstr2(i:i)  = ' '
         newstr(i:i) = ' '
      ENDDO
      ns = llen(subs)

C     Substitute values from file1 into file2 at the markers ...
      OPEN(UNIT=25,FILE=file1(1:llen(file1)),STATUS='OLD',ERR=8)
      OPEN(UNIT=26,FILE=file2(1:llen(file2)),STATUS='OLD',ERR=7)
1     READ(25,'(A)',END=5) cstr1
      nc1 = llen(cstr1)
2     READ(26,'(A)',END=4)  cstr2
      IF (comm.EQ.1.AND.cstr2(1:1).EQ.'#') GOTO 2
      nc2 = llen(cstr2)
      j = 0
      i = 1
3     IF (cstr2(i:(i+ns-1)).NE.subs(1:ns)) THEN
         j = j + 1
         IF (j.GT.1024) GOTO 6
         newstr(j:j) = cstr2(i:i)
         i = i + 1
      ELSE
         j = j + 1
         IF ((j+nc1-1).GT.1024) GOTO 6
         newstr(j:(j+nc1-1)) = cstr1(1:nc1)
         j = llen(newstr)
         i = i + ns
      ENDIF
      IF (i.LE.nc2) GOTO 3

      WRITE(6,'(A)') newstr(1:llen(newstr))
         
      DO i=1,1024
         cstr2(i:i)  = ' '
         newstr(i:i) = ' '
      ENDDO
      GOTO 2
         
4     REWIND(UNIT=26)
      DO i=1,1024
         cstr1(i:i)  = ' '
      ENDDO
      GOTO 1
      
5     CLOSE(UNIT=26)
      CLOSE(UNIT=25)
      GOTO 9

C     Error handling 
6     WRITE(6,'(A)')'ERROR: output line exceeds 1024 characters!'
      GOTO 9
7     WRITE(6,'(A)')
     +     'ERROR: could not open file \"'//file2(1:llen(file2))//'\"!'
      GOTO 9
8     WRITE(6,'(A)')
     +     'ERROR: could not open file \"'//file1(1:llen(file1))//'\"!'

9     RETURN
      END


C ----------------------------------------------------------------------

      SUBROUTINE argblkcp(file1,file2,subs,ntem,ierr)

C @(#)subroutine argblkcp: loop over the lines with replacement strings
C @(#)        in file1 and replace marker strings 'subs' per block of
C @(#)        'ntem' lines in file2 by the replacement strings.
      INTEGER	ierr, llen, ns, i,j,k, nc1, nc2, ntem, NN
      CHARACTER*80 file1, file2, subs
      CHARACTER*1024 cstr1, cstr2, newstr

C     Initialize...
      DO i=1,1024
         cstr1(i:i)  = ' '
         cstr2(i:i)  = ' '
         newstr(i:i) = ' '
      ENDDO
      ns = llen(subs)

C     Substitute values from file1 into file2 at the markers...
      OPEN(UNIT=25,FILE=file1(1:llen(file1)),STATUS='OLD',ERR=9)
      OPEN(UNIT=26,FILE=file2(1:llen(file2)),STATUS='OLD',ERR=8)
      NN = 0
1     READ(25,'(A)',END=4) cstr1
      NN = NN + 1

      nc1 = llen(cstr1)
      DO k=1,ntem
         READ(26,'(A)',END=5)  cstr2
         nc2 = llen(cstr2)
         j = 0
         i = 1
3        IF (cstr2(i:(i+ns-1)).NE.subs(1:ns)) THEN
            j = j + 1
            IF (j.GT.1024) GOTO 7
            newstr(j:j) = cstr2(i:i)
            i = i + 1
         ELSE
            j = j + 1
            IF ((j+nc1-1).GT.1024) GOTO 7
            newstr(j:(j+nc1-1)) = cstr1(1:nc1)
            j = llen(newstr)
            i = i + ns
         ENDIF
         IF (i.LE.nc2) GOTO 3
            
         WRITE(6,'(A)') newstr(1:llen(newstr))

         DO i=1,1024
            cstr2(i:i)  = ' '
            newstr(i:i) = ' '
         ENDDO
      ENDDO
            
      DO i=1,1024
         cstr1(i:i)  = ' '   
      ENDDO 
      GOTO 1


C     Test for error on exit loop...
4     READ(26,'(A)',END=6)
      WRITE(6,'(A)')'ERROR: file \"'//file2(1:llen(file2))//'\" has'//
     +          ' more lines than file \"'//file1(1:llen(file1))//'\"!'
      GOTO 10
5     WRITE(6,'(A)')'ERROR: file \"'//file1(1:llen(file1))//'\" has'//
     +          ' more lines than file \"'//file2(1:llen(file2))//'\"!'
      GOTO 10

C     Exit from loop was valid...
6     CLOSE(UNIT=26)
      CLOSE(UNIT=25)
      GOTO 10

C     Error handling...
7     WRITE(6,'(A)')'ERROR: output line exceeds 1024 characters!'
      GOTO 10
8     WRITE(6,'(A)')
     +      'ERROR: could not open file \"'//file2(1:llen(file2))//'\"!'
      GOTO 10
9     WRITE(6,'(A)')
     +      'ERROR: could not open file \"'//file1(1:llen(file1))//'\"!'
      
10    RETURN
      END


C ----------------------------------------------------------------------

      SUBROUTINE arglincp(file1,file2,subs,ierr)

C @(#)subroutine arglincp: loop over the lines with replacement strings
C @(#)        in file1 and replace marker strings 'subs' in the corres-
C @(#)        ponding line in file2 by the replacement strings.
      CHARACTER*80 file1, file2, subs
      CHARACTER*1024 cstr1, cstr2, newstr
      INTEGER   ierr, llen, ns, i,j, nc1, nc2

C     Initialize...
      DO i=1,1024
         cstr1(i:i)  = ' '
         cstr2(i:i)  = ' '
         newstr(i:i) = ' '
      ENDDO
      ns = llen(subs)

C     Substitute values from file1 into file2 at the markers...
      OPEN(UNIT=25,FILE=file1(1:llen(file1)),STATUS='OLD',ERR=97)
      OPEN(UNIT=26,FILE=file2(1:llen(file2)),STATUS='OLD',ERR=96)
1     READ(26,'(A)',END=5) cstr2
      READ(25,'(A)',END=10) cstr1
      nc1 = llen(cstr1)
      nc2 = llen(cstr2)
      j = 0
      i = 1
2     IF (cstr2(i:(i+ns-1)).NE.subs(1:ns)) THEN
         j = j + 1
         IF (j.GT.1024) GOTO 95
         newstr(j:j) = cstr2(i:i)
         i = i + 1
      ELSE
         j = j + 1
         IF ((j+nc1-1).GT.1024) GOTO 95
         newstr(j:(j+nc1-1)) = cstr1(1:nc1)
         j = llen(newstr)
         i = i + ns
      ENDIF
      IF (i.LE.nc2) GOTO 2
      WRITE(6,'(A)') newstr(1:llen(newstr))
         
      DO i=1,1024
         cstr1(i:i)  = ' '
         cstr2(i:i)  = ' '
         newstr(i:i) = ' '
      ENDDO
      GOTO 1


C     Test for error on exit loop...
5     READ(25,'(A)',END=11) cstr1
      WRITE(6,'(A)')'ERROR: file \"'//file1(1:llen(file1))//'\" has'//
     +          ' more lines than file \"'//file2(1:llen(file2))//'\"!'
      GOTO 99   
10    READ(26,'(A)',END=11) cstr2
      WRITE(6,'(A)')'ERROR: file \"'//file2(1:llen(file2))//'\" has'//
     +          ' more lines than file \"'//file1(1:llen(file1))//'\"!'
      GOTO 99

C     Exit from loop was valid...
11    CLOSE(UNIT=25)
      CLOSE(UNIT=26)
      GOTO 99

C     Error handling...
95    WRITE(6,'(A)')'ERROR: output line exceeds 1024 characters!'
      GOTO 99
96    WRITE(6,'(A)')
     +      'ERROR: could not open file \"'//file2(1:llen(file2))//'\"!'
      GOTO 99
97    WRITE(6,'(A)')
     +      'ERROR: could not open file \"'//file1(1:llen(file1))//'\"!'
      GOTO 99

99    RETURN
      END


C ----------------------------------------------------------------------

      SUBROUTINE manual()

      CHARACTER*80 mfile, cline
      INTEGER	llen

      mfile = '/iraf/extern/rjtools_2.12/src/arg.man'
      OPEN(UNIT=27,FILE=mfile(1:llen(mfile)),STATUS='OLD',ERR=3)
1     READ(27,'(A)',END=2) cline
      WRITE(6,'(A)') cline
      GOTO 1
2     CLOSE(UNIT=27)
      RETURN

3     WRITE(6,'(A)')'ERROR: could not open manual. Recompile \"arg\" '//
     +'or run \"man arg\"'
      WRITE(6,'(A)')'and see whether the package is installed properly.'

      RETURN
      END

C ----------------------------------------------------------------------
          
      SUBROUTINE clnarg(narg)
      
C @(#)Subroutine: clnarg
C @(#)   Count and return the number of command line arguments.
      INTEGER   narg
      
      narg = iargc()
      
      RETURN
      END


      SUBROUTINE clargs(index,param,ierr)

C @(#)Subroutine: clargs
C @(#)   Read a specified command line argument into a string variable
      INTEGER   index, ierr
      CHARACTER*80 cstr, param
      
      IF (ierr.NE.0) RETURN
         
      CALL getarg(index,cstr)
      READ(cstr,'(A)',ERR=2) param
      RETURN

2     ierr = 1
      RETURN
      END
