procedure getregion (image, output, format)

string	image	=""	{prompt="Name of input image"}
string	output	=""	{prompt="Name of output region file"}
string	format	="full"	{prompt="Output format",enum="full|basic|x|y"}
real	z1	=INDEF	{prompt="Lower cut level"}
real	z2	=INDEF	{prompt="Upper cut level"}
bool    redispl =yes    {prompt="(Re)display image first?"}
bool	imageid =no	{prompt="Prepend image name to region string?"}
bool	boxcen	=no	{prompt="Expand box center to rectangular region?"}
int	boxxsize=512	{prompt="X size of box region if boxcen=yes"}
int	boxysize=512	{prompt="Y size of box region if boxcen=yes"}
bool	chkbound=no	{prompt="Check pixel region bounds?"}
bool    append  =no     {prompt="Append to existing output file?"}
bool	savetmp	=no	{prompt="Save \"tvmark\" log- and command-files?"}
bool	verbose	=yes	{prompt="Verbose messages?"}

# @(#) task getregion                Author: R.A. Jansen -- June 19 1995
# @(#)
# @(#) task to mark rectangular regions on the image display by pointing
# @(#) at two diametrically opposite corners of the region  (e.g., lower
# @(#) left and upper right corner) if 'boxcen=no',  or at its center if
# @(#) 'boxcen=yes',  or at the two extremes if one of the range formats
# @(#) is specified.  The order in which these corners are marked doesnt
# @(#) matter.  The region coordinates are appended to a regions file if
# @(#) that file already exists and 'append=yes', in one of four suppor-
# @(#) ted formats ("basic"|"full"|"x"|"y")  where the image name/ID may
# @(#) be prepended if  'imageid=yes'.  Marked regions are  overlayed on
# @(#) the image display.
# @(#)
# @(#) Used 'rjtools' tasks: 'chkimg','rpparse','rpbuild' and 'xdisplay'
# @(#)
# @(#) Feb  6 1996 -- Implemented region expansion to square box given a
# @(#)                center position. Added option to prepend the image
# @(#)                ID to the region string. To distinguish normal and
# @(#)                box expansion modes, a box is drawn red. RAJ.
# @(#) Jun  9 1997 -- General rewrite of this task:  uses tvmark instead
# @(#)                of scan to read cursor.  Added formats "x" and "y"
# @(#)                to mark a set of x- or y- ranges (eg "2:14,21:28")
# @(#)                Re-displaying of the image may be suppressed,  and
# @(#)                output may be appended to existing file. RAJ.
# @(#) Jun 14 2000 -- Added support for multi-extension / N-dim. images.
# @(#) Apr  9 2001 -- Generalized expansion of boxes to rectangular box.
# @(#)                Added pixel bounds checking switch 'chkbound'. RAJ
# @(#) Feb 19 2002 -- Added verbose switch. RAJ.
# @(#) Jul 12 2003 -- imroot->impath//imroot after change in chkimg. RAJ
# @(#) Aug  8 2003 -- Cleaned up code; added tests, added support for  c
# @(#)		      key to mark a single pixel region (cosmic);  wrote
# @(#)                help file. RAJ

begin
	string	cimg, cout, cfm, cimr, ctyp, cpln, csec
	string	cstr, ocmd, cmd, key, arg, val
	int	xsiz, ysiz, nc, nr, dx1, dx2, dy1, dy2
	int	x, y, x0, y0, t
	file	tmptvmark, tmptvcmd
	real	zlo, zhi
	bool	new

#	Initialize variables...
	cimg	= image
	cout	= output
	cfm	= format
	zlo	= z1
	zhi	= z2
	xsiz	= boxxsize
	ysiz	= boxysize
	cpln	= ""
	csec	= ""
	if ( cfm == "x" || cfm == "y" ) boxcen = no

#	Check whether tv package is loaded...
	if ( !defpac("images") ) { images }
	if ( !defpac("tv") ) { tv }

#	Check whether image exists ...
	chkimg (cimg, "access", imtype="", verbose=yes)
	if (!chkimg.ok) { return }
	cimr = chkimg.impath//chkimg.imroot
	ctyp = chkimg.imtype

#	Check whether a region or extension was specified. 'chkimg'
#	called 'rpparse', so let's use its output...
	if ( rpparse.pln > 0 ) { cpln = "["//str(rpparse.pln)//"]" }
	rpbuild (rpparse.xlo, rpparse.xhi, rpparse.ylo, rpparse.yhi)
	if ( rpbuild.imrpstr != "[*,*]" ) { csec = rpbuild.imrpstr }
	cimg = cimr//ctyp//cpln//csec

#	Determine the image size ...
	hselect (cimr//ctyp//cpln, "i_naxis1", "yes") | scan (nc)
	hselect (cimr//ctyp//cpln, "i_naxis2", "yes") | scan (nr)
	if ( boxcen && ( xsiz > nc || ysiz > nr ) ) {
	   print ("ERROR: boxsize "//str(xsiz)//"x"//str(ysiz)//" exceeds size of image "//str(nc)//"x"//str(nr)//"!")
	   return
	}

#	Check whether output file is specified by an extension only ...
	if ( substr(cout,1,1) == "." ) { cout = cimr//cout }

#	Opening message...
	if (verbose) {
	   sysinfo() ; gdate(utc=no)
	   printf("GETREGION:  "//sysinfo.sysmsg//"  "//gdate.tdate//"\n")
	   printf("   image = "//cimg//" [%d x %d]\n", nc, nr)
	   printf("   output= "//cout//"  (format=\""//cfm//"\")\n")
	}

#	If an older version of the output file is present and append=no,
#	rename it...
	if ( access(cout) && !append ) {
	   if ( access(cout//"~") ) { delete (cout//"~", yes, verify=no) }
	   if (verbose) {
	      print ("   Saving older version of \""//cout//"\" --> \""//cout//"~\"...")
	   }
	   rename (cout, cout//"~", field="all")
	} else if ( access(cout) && verbose ) {
	   print ("   Appending to existing regions file "//cout//" ...")
	}

#	(Re)display image if 'redispl = yes' ...
	if (redispl) {
	   xdisplay (cimg, buff=1, z1=zlo, z2=zhi, lsig=3., hsig=15.,
		niter=1, zmode="linear", verbose=verbose)
	}

#	Compute appropriate half-sizes if 'boxcen=yes'...
	if (boxcen) {
	   if ( mod(xsiz,2) == 0 ) {
	      dx1 = xsiz/2 - 1
	      dx2 = xsiz/2
	   } else {
	      dx1 = int(xsiz/2)
	      dx2 = dx1
	   }
	   if ( mod(ysiz,2) == 0 ) {
	      dy1 = ysiz/2 - 1
	      dy2 = ysiz/2
	   } else {
	      dy1 = int(ysiz/2)
	      dy2 = dy1
	   }
	}

#	Temporary file to receive output from 'tvmark'...
	tmptvmark = mktemp("tvmark")

#	All preliminaries have been taken care of; now mark the regions...
	if (boxcen) {
	   if (verbose) {
	      print ("   Wait for the cursor cross to appear in the image display area,")
	      print ("   then mark all regions by hitting \"r\" at the region center, \"c\" to")
	      print ("   mark a single pixel, or \"b\" twice to mark an arbitrary rectangular")
	      print ("   region. Type \"q\" to quit.")
	   } else {
	      print ("\"r\" = box center, \"c\" = pixel, \"b\" twice = region, \"q\" = quit.")
	   }
	   sleep(1)

	   tvmark (frame=1, coor="", logfile=tmptvmark, autolog=yes,
		  outimage="", deletions="", commands="", mark="rectangle",
		  radii="1", lengths=str(xsiz)//" "//str(real(ysiz)/xsiz),
		  font="raster", color=203, label=no, number=no, nxoffset=0,
		  nyoffset=0, pointsize=0, txsize=0, tolerance=1.5,
		  interactive=yes, > "dev$null", >& "STDERR")

	} else if ( cfm == "x" || cfm == "y" ) {
	   if (verbose) {
	      print ("   Wait for the cursor cross to appear in the image display area,")
	      print ("   then mark all ranges by hitting \".\" at the extremes of each range.")
	      print ("   Hit \"q\" to quit.")
	   } else {
	      print ("\".\" twice = range, \"q\" = quit")
	   }
	   sleep(1)

	   tvmark (frame=1, coor="", logfile=tmptvmark, autolog=yes,
		  outimage="", deletions="", commands="", mark="none",
		  radii="1", lengths="0",
		  font="raster", color=203, label=no, number=no, nxoffset=0,
		  nyoffset=0, pointsize=1, txsize=1, tolerance=1.5,
		  interactive=yes, > "dev$null", >& "STDERR")

	} else {
	   if (verbose) {
	      print ("   Wait for the cursor cross to appear in the image display area,")
	      print ("   then mark all regions by hitting \"b\" once at each of two dia-")
	      print ("   gonally oposite corners to mark an arbitrary rectangular region,")
	      print ("   or \"c\" once to mark a single pixel. Hit \"q\" to quit.")
	   } else {
	      print ("\"b\" twice = region, \"c\" = pixel, \"q\" = quit.")
	   }
	   sleep(1)

	   tvmark (frame=1, coor="", logfile=tmptvmark, autolog=yes,
		  outimage="", deletions="", commands="", mark="plus",
		  radii="1", lengths="0",
		  font="raster", color=203, label=no, number=no, nxoffset=0,
		  nyoffset=0, pointsize=3, txsize=1, tolerance=1.5,
		  interactive=yes, > "dev$null", >& "STDERR")
	}

#	Loop over the output of tvmark, interpret the region commands
#	and format the regions marked according to 'format'...
	tmptvcmd = mktemp("tvmark")
	ocmd = ""
	list = tmptvmark
	while( fscan(list,x,y,key,cmd,arg,val) != EOF ) {
	   if ( x < 1 )  x = 1
	   if ( x > nc ) x = nc
	   if ( y < 1 )  y = 1
	   if ( y > nr ) y = nr
	   if ( cmd == "c" && cfm != "x" && cfm != "y" ) {
	      cstr = ""
	      print (x, y, "101  :color 206", >> tmptvcmd)
	      print (x, y, "101  b", >> tmptvcmd)
	      print (x, y, "101  b", >> tmptvcmd)
	      if ( cfm == "basic" ) {
		 if (imageid) cstr = cimr//ctyp//cpln//" "
		 printf(cstr//"%d %d %d %d\n", x, x, y, y, >> cout)
	      } else {
		 if (imageid) cstr = cimr//ctyp//cpln
		 printf(cstr//"[%d:%d,%d:%d]\n", x, x, y, y, >> cout)
	      }
	      ocmd = ""

	   } else if ( cmd == "c" ) {
	      print ("WARNING: Ignoring \"c\" command (\"format="//cfm//"\")! Continuing...")
	      ocmd = ""

	   } else if ( cmd == "r" && boxcen && cfm != "x" && cfm != "y" ) {
	      x0 = x - dx1
	      x  = x + dx2
	      y0 = y - dy1
	      y  = y + dy2
	      if (chkbound) {
		 if ( x0 < 1 ) { x  = x - x0 + 1 ; x0 = 1 }
		 if ( y0 < 1 ) { y  = y - y0 + 1 ; y0 = 1 }
		 if ( x > nc ) { x0 = x0 - x + nc ; x  = nc }
		 if ( y > nc ) { y0 = y0 - y + nr ; y  = nr }
	      }
	      cstr = ""
	      print (x0, y0, "101  :color 204", >> tmptvcmd)
	      print (x0, y0, "101  b", >> tmptvcmd)
	      print (x, y, "101  b", >> tmptvcmd)
	      if ( cfm == "basic" ) {
		 if (imageid) cstr = cimr//ctyp//cpln//" "
		 printf(cstr//"%d %d %d %d\n", x0, x, y0, y, >> cout)
	      } else {
		 if (imageid) cstr = cimr//ctyp//cpln
		 printf(cstr//"[%d:%d,%d:%d]\n", x0, x, y0, y, >> cout)
	      }
	      ocmd = ""

	   } else if ( cmd == "r" ) {
	      if ( cfm != "x" && cfm != "y" ) {
		 print ("WARNING: Ignoring \"r\" command (\"boxcen="//str(boxcen)//"\")! Continuing...")
	      } else {
		 print ("WARNING: Ignoring \"r\" command (\"format="//cfm//"\")! Continuing...")
	      }
	      ocmd = ""

	   } else if ( cmd == "b" && cfm != "x" && cfm != "y" ) {
	      if ( ocmd == "" ) {
		 cstr = ""
		 ocmd = cmd
		 x0   = x
		 y0   = y
	      } else if ( ocmd == "b" || ocmd == "c" ) {
		 if ( x0 > x ) { t = x0 ; x0 = x ; x = t }
		 if ( y0 > y ) { t = y0 ; y0 = y ; y = t }
	         print (x0, y0, "101  :color 205", >> tmptvcmd)
		 print (x0, y0, "101  b", >> tmptvcmd)
		 print (x, y, "101  b", >> tmptvcmd)
		 if ( cfm == "basic" ) {
		    if (imageid) cstr = cimr//ctyp//cpln//" "
		    printf(cstr//"%d %d %d %d\n", x0, x, y0, y, >> cout)
		 } else {
		    if (imageid) cstr = cimr//ctyp//cpln
		    printf(cstr//"[%d:%d,%d:%d]\n", x0, x, y0, y, >> cout)
		 }
		 ocmd = ""
	      } else {
		 print ("WARNING: Ignoring invalid command sequence \""//ocmd//"\",\"b\"! Continuing...")
		 ocmd = ""
	      }

	   } else if ( cmd == "b" ) {
	      print ("WARNING: Ignoring \"b\" command (\"format="//cfm//"\")! Continuing...")
	      ocmd = ""

	   } else if ( cmd == "." && ( cfm == "x" || cfm == "y" ) ) {
	      if ( ocmd == "" ) {
		 cstr = ""
		 ocmd = cmd
		 x0   = x
		 y0   = y
		 new  = yes
		 print (x0, y0, "0  :color 206", >> tmptvcmd)
	      } else if ( ocmd == "." ) {
		 if (new) {
		    if ( cstr != "" ) cstr = cstr//","
		    new = no
		 } else {
		    new = yes
		 }
		 if ( x0 > x ) { t = x0 ; x0 = x ; x = t }
		 if ( y0 > y ) { t = y0 ; y0 = y ; y = t }
		 if ( cfm == "x" ) {
		    cstr = cstr//x0//":"//x
		    print (x0,y," 206  .", >> tmptvcmd)
		    print (x0,(y-3)," 206  s", >> tmptvcmd)
		    print (x0,(y+3)," 206  s", >> tmptvcmd)
		    print (x0,y," 206  s", >> tmptvcmd)
		    print (x,y," 206  s", >> tmptvcmd)
		    print (x,(y-3)," 206  s", >> tmptvcmd)
		    print (x,(y+3)," 206  s", >> tmptvcmd)
		    print (x0,(y+1)," 206  .", >> tmptvcmd)
		    print (x0,(y-2)," 206  s", >> tmptvcmd)
		    print (x0,(y+4)," 206  s", >> tmptvcmd)
		    print (x0,(y+1)," 206  s", >> tmptvcmd)
		    print (x,(y+1)," 206  s", >> tmptvcmd)
		    print (x,(y-2)," 206  s", >> tmptvcmd)
		    print (x,(y+4)," 206  s", >> tmptvcmd)
		 } else {
		    cstr = cstr//y0//":"//y
		    print (x,y0," 206  .", >> tmptvcmd)
		    print ((x-3),y0," 206  s", >> tmptvcmd)
		    print ((x+3),y0," 206  s", >> tmptvcmd)
		    print (x,y0," 206  s", >> tmptvcmd)
		    print (x,y," 206  s", >> tmptvcmd)
		    print ((x-3),y," 206  s", >> tmptvcmd)
		    print ((x+3),y," 206  s", >> tmptvcmd)
		    print ((x+1),y0," 206  .", >> tmptvcmd)
		    print ((x-2),y0," 206  s", >> tmptvcmd)
		    print ((x+4),y0," 206  s", >> tmptvcmd)
		    print ((x+1),y0," 206  s", >> tmptvcmd)
		    print ((x+1),y," 206  s", >> tmptvcmd)
		    print ((x-2),y," 206  s", >> tmptvcmd)
		    print ((x+4),y," 206  s", >> tmptvcmd)
		 }
		 ocmd = cmd

	      } else {
		 print ("WARNING: Ignoring invalid command sequence \""//ocmd//"\",\".\"! Continuing...")
		 ocmd = ""
	      }

	   } else if ( cmd == "." ) {
	      print ("WARNING: Ignoring \".\" command (\"format="//cfm//"\")! Continuing...")
	      ocmd = ""

	   } else {
	      print ("WARNING: Skipping unrecognized command \""//cmd//"\"! Continuing...")
	      ocmd = ""
	   }
	}
	list = ""

	print ("0  0  0  q", >> tmptvcmd)
	if ( cfm == "x" || cfm == "y" ) print (cstr, >> cout)

#	Now, redraw the regions or ranges using their segregated colors:
#	'b'-->green, 'r'--> red, '.'-->blue. If everything went OK, every
#	white region mark should now be colored in; if not then better
#	check what went wrong...
	tvmark (frame=1, coor="", logfile="", autolog=no, outimage="",
		deletions="", commands=tmptvcmd, mark="point", radii="0", 
		lengths="2", font="raster", color=203, label=no,
		number=no, nxoffset=0, nyoffset=0, pointsize=1, txsize=1,
		tolerance=1.5, interactive=no, > "dev$null", >& "STDERR")

	sleep(1)

#	Clean-up; optionally saving the tvmark output file...
	if (savetmp) {
	   if ( access(cimr//".tvm") ) delete (cimr//".tvm", yes, verify-)
	   if ( access(cimr//".tvc") ) delete (cimr//".tvc", yes, verify-)
	   rename (tmptvmark, cimr//".tvm", field="all")
	   rename (tmptvcmd, cimr//".tvc", field="all")
	   if (verbose) {
	      print ("# GETREGION Finished. Saved \"tvmark\" files as \""//cimr//".tvm\" and \"-.tvc\".")
	   }
	} else {
	   delete (tmptvmark//","//tmptvcmd, yes, verify-)
	   if (verbose) print ("# GETREGION Finished")
	}

end
