procedure tvellips (image, xcen, ycen, ellipt, posang, drad)

string	image	=""	{prompt="Image name"}
real	xcen	=INDEF	{prompt="Center position x (pixels)"}
real	ycen	=INDEF	{prompt="Center position y (pixels)"}
real	ellipt	=0.	{prompt="Ellipticity (1-b/a)"}
real	posang	=0.	{prompt="Position angle (wrt. x-axis)"}
string	drad	="1."	{prompt="Comma-separated list of semi-major axis radii"}
string	color	="205"	{prompt="Color to use for ellipse"}
real	zlow	=INDEF	{prompt="Lower cut level (display)"}
real	zhigh	=INDEF	{prompt="Upper cut level"}
bool	redispl =yes	{prompt="(Re)display image first?"}
bool	calconly=no	{prompt="Do not display, only compute ellipses?"}
string	calcfil	=""	{prompt="Name of output file"}
string	calcfmt	="full"	{prompt="Format of output file (basic|full|tvmark)",
			 enum="basic|full|tvmark"}
bool	calcpid	=yes	{prompt="Prepend image name in output file?"}

# @(#) task tvellips                  Author: R.A. Jansen -- Apr 22 1997
# @(#) 
# @(#) task to compute and overlay  a set of ellipses  of arbitrary size
# @(#) and orientation,  as specified by 'ellipt' and 'posang' and a set
# @(#) of semi-major axis radii 'drad', on a displayed image.  This task
# @(#) only computes the ellipses but does not display  the results when
# @(#) 'calconly=yes'.  In either case,  the resulting  "tvmark" command
# @(#) file is stored in 'calcfil',  as long as  'calcfil'  is specified.
# @(#) This task was adapted from an earlier one by M. Franx.
# @(#)
# @(#) Calls 'rjtools' tasks: 'chkimg', 'rpparse', 'rpbuild', 'cparse',
# @(#)                       and 'xdisplay'
# @(#)
# @(#) Jun 30 2000 -- Added support for multi-extension / N-dim. images.
# @(#) Aug 10 2003 -- Added support for differently colored ellipses, by
# @(#)                makin 'color' a comma-separated list of colors and
# @(#)                adding  :color statement to 'tvmark' command file;
# @(#)                wrote help file. RAJ.

begin
	string	cimg, crad, clr, cfil, cfmt
	string	cimr, ctyp, cpln, csec, ostr, cstr
	int	nc, nr, nc0, nr0, nell, i, ns, is, iclr, oclr, ib
	real	cx, cy, eps, pa, rlo, rhi, da
	real	pi, cospa, sinpa, phi, cosphi, sinphi, x, y

#	Initialize variables...
	cimg  = image
	cx    = xcen
	cy    = ycen
	eps   = ellipt
	pa    = posang
	crad  = drad
	clr   = color
	rlo   = zlow
	rhi   = zhigh
	cfil  = calcfil
	cfmt  = calcfmt
	nc0   = 1
	nr0   = 1
	pi    = 3.141592654

#	Check whether tv package is loaded ...
	if ( !calconly && !defpac("images") ) { images }
	if ( !calconly && !defpac("tv") ) { tv }

#       Check whether input image exists ...
        chkimg (cimg, "access", imtype="", verbose=yes)
        if (!chkimg.ok) { return }
        cimr = chkimg.impath//chkimg.imroot
	ctyp = chkimg.imtype
	cpln = ""
	csec = ""

#       Check whether a region or image extension was specified; 'chkimg'
#       called 'rpparse', so let's use its output ...
        if ( rpparse.pln > 0 ) { cpln = "["//str(rpparse.pln)//"]" }
	rpbuild (rpparse.xlo, rpparse.xhi, rpparse.ylo, rpparse.yhi)
        if ( rpbuild.imrpstr != "[*,*]" ) { csec = rpbuild.imrpstr }
	cimg = cimr//ctyp//cpln//csec

#	If both 'xcen' and 'ycen' are undefined, use image center or
#	center of specified image region...
	hselect (cimg, "i_naxis1", "yes") | scan (nc)
	hselect (cimg, "i_naxis2", "yes") | scan (nr)
	if ( cx == INDEF && cy == INDEF ) {
	   if ( csec != "" ) {
	      cx  = rpparse.xcen
	      cy  = rpparse.ycen
	      nc0 = rpparse.xlo
	      nr0 = rpparse.ylo
	   } else {
	      cx  = nint(0.5*nc)
	      cy  = nint(0.5*nr)
	   }
	} else if ( cx == INDEF || cy == INDEF ) {
	   error (1,"Ellipse centers not specified (INDEF)!")
	} else if ( csec != "" ) {
	   nc0 = rpparse.xlo
	   nr0 = rpparse.ylo
	}

#       Check whether 'calcfil' is specified by an extension only ...
	if ( calconly && cfil == "" ) error (1,"'calcfil'=\"\", while 'calconly'=yes!")
	if ( cfil != "" ) {
	   if ( substr(cfil,1,1) == "." ) { cfil = cimr//cfil }
	   # Check whether the file already exists...
	   chkimg (cfil, "noaccess", imtype="file", verbose=yes)
	   if (!chkimg.ok) { return }
	}

#       Check string with semi-major axis radii...
	cparse (crad, delim=",")
	nell = cparse.nfields
	if ( nell == 0 ) error (1,"No semi-major axis radii specified!")

#       (Re)display input image if 'redispl==yes'...
	if ( redispl && !calconly ) {
	   xdisplay (cimg, buff=1, z1=rlo, z2=rhi, zmode="linear", era+, verb-)
	}

#       Loop over all semi-major axis radii and compute ellipses ...
	oclr = -1
	for ( i = 1 ; i <= nell ; i = i + 1 ) {

	   cparse (crad, delim=',', nselect=i)
	   # Make sure we have a valid radius...
	   ib = stridx("abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ~!@#$%^&*_-=+[]|;:,<>?/'`",cparse.cfield)
	   if ( ib > 0 ) {
	      print ("WARNING: invalid radius \""//cparse.cfield//"\"! Continuing...")
              da = -1.0*pi
	   } else {
	      da = real(cparse.cfield)
	   }
	   ns    = nint(2.0*pi*da) + 1
	   cospa = cos(pa*pi/180.)
	   sinpa = sin(pa*pi/180.)
	   cparse (clr, delim=",", nselect=i)

	   # Make sure we have a valid numeric color...
	   ib = stridx("abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ~!@#$%^&*_-=+[]|;:,<>?/'`",cparse.cfield)
	   if ( ib > 0 ) {
	      print ("WARNING: invalid color \""//cparse.cfield//"\"! Using 255 instead...")
	      iclr = 255
	   } else {
	      iclr = int(cparse.cfield)
	      if ( iclr > 255 ) iclr = 255
	      if ( iclr <   0 ) iclr = 0
	   }

	   ostr = ""
	   for ( is = 0 ; is <= ns ; is = is + 1 ) {
	      phi = 2.0*pi*is/ns
	      cosphi = cos(phi)
	      sinphi = sin(phi)
	      x = cx + (cospa*cosphi*da) - (sinpa*sinphi*da*(1.0-eps))
	      y = cy + (sinpa*cosphi*da) + (cospa*sinphi*da*(1.0-eps))
	      # Check pixel bounds...
	      if ( x >= 1 && x <= (nc+nc0) && y >= 1 && y <= (nr+nr0) ) {
		 if ( iclr != oclr ) {
		    printf("%.2f %.2f 101 :color %d\n", x,y,iclr, >> "tmptvcmd")
	            printf("%.2f %.2f 101 .\n", x,y, >> "tmptvcmd")
		    oclr = iclr
		 } else {
		    printf("%.2f %.2f 101 s\n", x,y, >> "tmptvcmd")
	         }
	         if ( cfil != "" && cfmt != "tvmark" ) {
		    if ( cfmt == "basic" ) {
		       cstr = " "//str(nint(x))//" "//str(nint(x))//" "//str(nint(y))//" "//str(nint(y))
		    } else {
		       printf("[%d:%d,%d:%d]\n",nint(x),nint(x),nint(y),nint(y)) | scan (cstr)
		    }
		    if ( calcpid ) { cstr = cimr//ctyp//cstr }
		    if ( cstr != ostr ) { print (cstr, >> cfil) }
		    ostr = cstr
	         }
	      }
	   }

	# goto next ellipse
	}

#	If 'format="tvmark"', save the 'tvmark' command file...
	if ( cfil != "" && cfmt == "tvmark" ) {
	   copy ("tmptvcmd", cfil, verbose=no)
	}

#	Overlay ellipse on displayed image...
	if ( !calconly ) {
	   tvmark (frame=1, coords="", logfile="", autolog=no, outimage="",
		deletions="", commands="tmptvcmd", mark="point", radii="0",
		lengths="2", font="raster", color=iclr, label=no,
		number=no, nxoffset=0, nyoffset=0, pointsize=1, txsize=1,
		tolerance=1.5, interactive=no, > "dev$null", >& "STDERR")
	}

#	Clean-up...
	delete ("tmptvcmd", yes, verify=no)
      
end
