procedure xdisplay (image)

string	image	=""	{prompt="image to display"}
int	buff	=1	{prompt="frame buffer to use for display"}
real	z1	=INDEF	{prompt="fix min level to display to (DN)"}
real	z2	=INDEF	{prompt="fix max level to display to (DN)"}
real	lsig	=3.	{prompt="try setting min level to lsig below mode)"}
real	hsig	=15.	{prompt="try setting max level to hsig above mode)"}
int	niter	=1	{prompt="number of rejection iterations"}
string	zmode	="linear"	{enum="linear|log",prompt="transformation"}
real	cenx	=0.5	{prompt="x center (fraction of full image)"}
real	ceny	=0.5	{prompt="y center (fraction of full image)"}
real	sizx	=1.0	{prompt="display window horizontal size"}
real	sizy	=1.0	{prompt="display window vertical size"}
real	zoom	=1.	{min=0.125,max=8.,prompt="magnification factor"}
bool	erase	=yes	{prompt="erase frame before loading image?"}
bool	verbose	=no	{prompt="verbose messages?"}
real	tz1	=0.	{prompt="minimum greylevel actually displayed"}
real	tz2	=0.	{prompt="maximum greylevel actually displayed"}
real	tmed	=0.	{prompt="mediam pixel value after clipping"}
real	tsig	=0.	{prompt="approximate sigma after clipping"}
real	tmin	=0.	{prompt="minimum pixel value after clipping"}
real	tmax	=0.	{prompt="maximum pixel value after clipping"}

# @(#) task xdisplay                  Author: R.A. Jansen -- Jul 31 1997
# @(#)
# @(#) User interface to task 'display', with a somewhat better estimate
# @(#) of the scale and range of levels to display,  particularly in the
# @(#) case of images of extended objects.  But even for stellar images,
# @(#) by increasing the default values of 'lsig' and 'hsig' (e.g.,  5.0
# @(#) and 25.), good results are obtained. Parameters 'lsig' and 'hsig'
# @(#) define  sigma clipping levels  that are applied  after an initial
# @(#) iteration to determine the image statistics.  The resulting range
# @(#) of pixel values is compared to the values specified for 'z1' and/
# @(#) or 'z2' if they differ from INDEF,  and are then  further tweaked
# @(#) based on the statistics in a smaller region 1/4 the size centered
# @(#) on the region that is to be displayed.  The rationale for this is
# @(#) that usually the object(s) of interest are nearer the center than
# @(#) the border of the display region.
# @(#)
# @(#) Calls 'rjtools' tasks: 'chkimg','rpparse','rpbuild' and 'imgstat'
# @(#)
# @(#) Jun 14 2000 -- Added support for multi-extension/N-dim images.RAJ.
# @(#) May 11 2001 -- Added 'zoom' and 'buff' parameters. RAJ.
# @(#) Jul 12 2003 -- imroot->impath//imroot after change in chkimg. RAJ. 
# @(#) Jul 18 2003 -- Added 't*' parameters to return display levels 
# @(#)		      actually used as task parameters; wrote help.  RAJ.
# @(#) Jul 12 2007 -- Fixed bug in call to 'rpbuild' that caused message
# @(#)                "No pixels with values between .. and ..!" for MEF
# @(#)                images with extension/plane specifier. RAJ.

begin
	string	cimg, ctrn, csec, cmag
	real	zlo, zhi, slo, shi, sm, ravg, rsig, rlo, rhi, xc, yc
	int	ifrm, ni, nc, nr, i
	bool	bclr

#	Initialize variables...
	cimg	= image
	ifrm	= buff
	zlo	= z1
	zhi	= z2
	slo	= lsig
	shi	= hsig
	ni	= niter
	ctrn	= zmode
	bclr	= erase

#	Check input to see whether any sigma levels are given...
	if ( slo == INDEF ) { slo = 3. }
	if ( shi == INDEF ) { shi = 15. }
	sm = max(slo,shi)

#	Check whether tv package is loaded...
	if ( !defpac("images") ) { images }
	if ( !defpac("tv") ) { tv }

#	Check whether image exists ...
	chkimg (cimg, "access", imtype="", verbose=yes)
	if (!chkimg.ok) { return }
	csec = chkimg.imrpstr
	cimg = chkimg.impath//chkimg.imroot//chkimg.imtype

#	Determin raw image statistics...
	imgstat (cimg//csec, lower=INDEF, upper=INDEF, verbose-)
	ravg = imgstat.mean
	rsig = imgstat.stddev
	rlo  = ravg - sm*rsig
	rhi  = ravg + sm*rsig

#	Determine image statistics after 'ni' rejection iterations and
#	set cut-levels...
	for ( i = 1 ; i <= ni ; i = i + 1 ) {
	   imgstat (cimg//csec, lower=rlo, upper=rhi, verbose-)
	   ravg = imgstat.rmode
	   rsig = imgstat.stddev
	   rlo	= ravg - (slo/sqrt(i-0.5))*rsig
	   rhi  = ravg + (shi/sqrt(i-0.5))*rsig
	}

#	Usually the object of interest is closer to the middle of the
#	image than it is to the image borders...
	hselect (cimg//csec, "i_naxis1", "yes") | scan (nc)
	hselect (cimg//csec, "i_naxis2", "yes") | scan (nr)
	rpparse (csec, verbose=no)
	xc = rpparse.xcen ; yc = rpparse.ycen
	if ( rpparse.xcen == INDEF ) { xc = 0.5*nc }
	if ( rpparse.ycen == INDEF ) { yc = 0.5*nr }
	rpbuild (int(xc-0.25*nc), int(xc+0.25*nc), int(yc-0.25*nr),
		int(yc+0.25*nr), pln=rpparse.pln)
	cmag = rpbuild.imrpstr
	imgstat (cimg//cmag, lower=rlo, upper=rhi, verbose-)

#	Fuzzy logic: tweak cut levels found...
	if ( rhi < rlo ) {
	   rhi = rlo + 3*rsig
	   rlo = rlo - rsig
	}
	if ( rhi > (imgstat.max+0.3*(imgstat.max-imgstat.min)) ) {
	   rhi = imgstat.max
	}
	if ( rlo < (imgstat.min-0.1*(imgstat.max-imgstat.min)) ) {
	   rlo = imgstat.min
	}

#	If either or both z1 and z2 were supplied, fix the cut levels to
#	the requested values:
	if ( zlo != INDEF ) { rlo = zlo }
	if ( zhi != INDEF ) { rhi = zhi }

	# Display image...
	if (verbose) {
	   printf("Displaying image \""//cimg//csec//"\":  z1=%.2f z2=%.2f\n",
		rlo, rhi)
	}
	display (cimg//csec, ifrm, erase=bclr, border_era=bclr, select_f=yes,
		repeat=no, fill=no, zscale=no, contrast=0.25, zrange=no,
		nsample=1000, xcenter=cenx, ycenter=ceny, xsize=sizx,
		ysize=sizy, xmag=zoom, ymag=zoom, order=0, z1=rlo, z2=rhi,
		ztrans=ctrn, lutfile="", >> "dev$null")

	# Return the display levels used as task parameters...
	image = cimg
	tz1   = rlo
	tz2   = rhi
	tmed  = imgstat.midpt
	tsig  = imgstat.stddev
	tmin  = imgstat.min
	tmax  = imgstat.max

end
